import { Request, Response } from "express";
import MemberService from "../../service/memberService";
import AwardService from "../../service/awardService";
import CommunicationTypeService from "../../service/communicationTypeService";
import ExecutivePositionService from "../../service/executivePositionService";
import crypto from "crypto";
import { type BinaryLike } from "node:crypto";

/**
 * @description get all data stored in the database as a single json object
 * @param req {Request} Express req object
 * @param res {Response} Express res object
 * @returns {Promise<*>}
 */
export async function getDatabaseExport(req: Request, res: Response): Promise<any> {
  const { secret } = req.body;
  try {
    if (!secret) {
      res.status(400).send('Empty encryption key');
      return;
    }
    const iv = await new Promise((resolve, reject) => {
      // create initialization vector
      crypto.randomBytes(16, (err, iv) => {
        if (err) {
          reject(err);
          return;
        }
        resolve(iv);
      });
    });

    let data: any = {};

    console.log('Exporting awards...');
    data.awards = await AwardService.getAll();
    console.log(`Exported ${data.awards.length} awards`);
    console.log('Exporting communication types...');
    data.communicationTypes = await CommunicationTypeService.getAll();
    console.log(`Exported ${data.communicationTypes.length} communicationTypes`);
    console.log('Exporting executivePositions...');
    data.executivePositions = await ExecutivePositionService.getAll();
    console.log(`Exported ${data.executivePositions.length} executivePositions`);
    console.log('Exporting members...');
    data.members = await MemberService.getAll();
    console.log(`Exported ${data.members.length} members`);

    console.log(`Encrypting data...`);
    // encrypt data
    const dataStr: string = JSON.stringify(data);
    let base64Key = crypto.createHash('sha256').update(String(secret)).digest('base64');
    const key = Buffer.from(base64Key, 'base64')
    let cipher = crypto.createCipheriv("aes-256-gcm", key, iv as BinaryLike);
    const encryptedData: string = cipher.update(dataStr, 'utf8', 'hex') + cipher.final('hex');

    res.json({
      data: encryptedData,
    });
  }
  catch(ex) {
    console.log(ex);
    res.status(500).send(ex.message ? ex.message : 'Exception!');
  }
}
